/*
 * This file is part of Skyward Hub.
 *
 * Skyward Hub is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * Skyward Hub is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * Skyward Hub. If not, see <https://www.gnu.org/licenses/>.
 *
 */

#include "Filter.h"

Filter::Filter() {}

Filter::Filter(Topic topic) : topic(topic) {}

Filter::Filter(Topic topic, QSet<QString> fields) : topic(topic), fields(fields)
{
}

bool Filter::operator==(const Filter& toCompare) const
{
    return toString() == toCompare.toString();
}

bool Filter::operator!=(const Filter& toCompare) const
{
    return !(*this == toCompare);
}

bool Filter::operator<(const Filter& toCompare) const
{
    return toString() < toCompare.toString();
}

void Filter::setTopic(Topic topic) { this->topic = topic; }

Topic Filter::getTopic() const { return topic; }

void Filter::addField(QString field) { fields.insert(field); }

bool Filter::removeField(QString field) { return fields.remove(field); }

QSet<QString> Filter::getFields() const { return fields; }

bool Filter::match(const Message& message) const
{
    return this->topic == message.getTopic();
}

bool Filter::filterMessage(Message& message) const
{
    // Check if the message matches the topic
    if (!match(message))
        return false;

    // If the filter specifies at leas one field, filter them
    if (fields.size() > 0)
        message.filterFields(fields);

    return true;
}

QString Filter::toString() const
{
    QString string;

    string += topic.toString();
    string += ":";

    if (!fields.isEmpty())
    {
        auto list = fields.values();
        for (int i = 0; i < list.size() - 1; i++)
            string += list[i] + ",";
        string += list.last();
    }
    else
    {
        string += "*";
    }

    return string;
}

QString Filter::toPrettyString() const
{
    QString string;

    string += topic.toString();
    string += ":";

    if (!fields.isEmpty())
    {
        auto list = fields.values();

        if (fields.size() > 1)
        {
            string += "\n";

            for (int i = 0; i < list.size() - 1; i++)
                string += "    " + list[i] + ",\n";
            string += "    " + list.last();
        }
        else
        {
            string += " " + list.last();
        }
    }
    else
    {
        string += "*";
    }

    return string;
}

Filter Filter::fromString(QString string)
{
    int idxCol = string.indexOf(':');

    auto filter = Filter(Topic(string.left(idxCol)));

    if (idxCol > 0)
    {
        auto right = string.right(string.size() - 1 - idxCol);

        for (auto field : right.split(','))
        {
            if (field.trimmed() != "*")
                filter.addField(field.trimmed());
        }
    }

    return filter;
}
